import java.applet.Applet;
import java.awt.BorderLayout;
import java.awt.image.BufferedImage;
import javax.media.j3d.*;
import javax.vecmath.*;
import com.sun.j3d.utils.geometry.*;
import com.sun.j3d.utils.image.TextureLoader;
import com.sun.j3d.utils.universe.*;

public class FreeWalk extends Applet
{
	private Canvas3D canvas;
	private SimpleUniverse universe;

	private Map map;
	private TransformGroup birdView;

	public void init()
	{
		canvas=new Canvas3D(SimpleUniverse.getPreferredConfiguration());
		universe=new SimpleUniverse(canvas);
		
		universe.getViewingPlatform().setNominalViewingTransform();
		universe.addBranchGraph(createSceneGraph());
		
		universe.getViewer().getView().setFrontClipPolicy(View.VIRTUAL_EYE);
		universe.getViewer().getView().setFrontClipDistance(0.5);
		universe.getViewer().getView().setBackClipPolicy(View.VIRTUAL_EYE);
		universe.getViewer().getView().setBackClipDistance(1000.0);
		
		setLayout(new BorderLayout());
		add(canvas,BorderLayout.CENTER);
	}

	private BranchGroup createSceneGraph()
	{
		BranchGroup root;
		PlatformGeometry platform;
	
		root=new BranchGroup();
		root.addChild(createLight());
		root.addChild(createTile());
		root.addChild(createBox());
		root.addChild(createBall());
		root.addChild(createCone());
		root.addChild(createBirdView());
		
		platform=new PlatformGeometry();
		platform.addChild(createMap());
		universe.getViewingPlatform().setPlatformGeometry(platform);
		
		return root;
	}

	private Node createLight()
	{
		Light light;
		BoundingSphere bounds;
	
		light=new PointLight(new Color3f(10.0f,10.0f,10.0f),
		                     new Point3f( 0.3f, 5.0f,10.0f),
		                     new Point3f( 0.1f, 0.1f, 0.0f));
		
		bounds=new BoundingSphere(new Point3d(),Double.POSITIVE_INFINITY);
		light.setInfluencingBounds(bounds);
		
		return light;
	}

	private Node createTile()
	{
		TransformGroup transform;
		Appearance appearance;
		Texture texture;
		Material material;
		Box tile;
	
		transform=new TransformGroup();
		
		appearance=new Appearance();
		texture=new TextureLoader("tile.jpg",this).getTexture();
		appearance.setTexture(texture);
		material=new Material();
		material.setLightingEnable(true);
		appearance.setMaterial(material);
		
		tile=new Box(5.0f,0.01f,5.0f,
		    Box.GENERATE_TEXTURE_COORDS | Box.GENERATE_NORMALS,appearance);
		transform.addChild(tile);
		
		return transform;
	}

	private Node createBox()
	{
		Transform3D translation;
		TransformGroup transform;
		Appearance appearance;
		Material material;
		Sphere self;
	
		translation=new Transform3D();
		translation.setTranslation(new Vector3d(0.0,0.3,0.0));
		transform=new TransformGroup(translation);
		
		appearance=new Appearance();
		material=new Material();
		material.setDiffuseColor(0.02f,0.01f,0.10f);
		material.setSpecularColor(0.02f,0.01f,0.10f);
		material.setLightingEnable(true);
		appearance.setMaterial(material);
		
		self=new Sphere(0.3f,Sphere.GENERATE_NORMALS,appearance);
		transform.addChild(self);
		
		return transform;
	}

	private Node createBall()
	{
		Transform3D translation;
		TransformGroup transform;
		Appearance appearance;
		Material material;
		Sphere ball;
	
		translation=new Transform3D();
		translation.setTranslation(new Vector3d(0.5,0.5,-1.5));
		transform=new TransformGroup(translation);
		
		appearance=new Appearance();
		material=new Material();
		material.setDiffuseColor(0.01f,0.10f,0.02f);
		material.setSpecularColor(0.01f,0.10f,0.02f);
		material.setLightingEnable(true);
		appearance.setMaterial(material);
		
		ball=new Sphere(0.5f,Sphere.GENERATE_NORMALS,appearance);
		transform.addChild(ball);
		
		return transform;
	}

	private Node createCone()
	{
		Transform3D translation;
		TransformGroup transform;
		Appearance appearance;
		Material material;
		Cone cone;
	
		translation=new Transform3D();
		translation.setTranslation(new Vector3d(-0.7,0.5,-0.5));
		transform=new TransformGroup(translation);
		
		appearance=new Appearance();
		material=new Material();
		material.setDiffuseColor(0.10f,0.02f,0.01f);
		material.setSpecularColor(0.10f,0.02f,0.01f);
		material.setLightingEnable(true);
		appearance.setMaterial(material);
		
		cone=new Cone(0.5f,1.0f,Sphere.GENERATE_NORMALS,appearance);
		transform.addChild(cone);
		
		return transform;
	}

	private Node createMap()
	{
		Transform3D translation1,translation2;
		TransformGroup transform;
		ViewPlatform platform;
	
		translation1=new Transform3D();
		translation1.setTranslation(new Vector3d(0.2,0.2,-1.0));
		map=new Map(
		    universe.getViewer().getView(),
		    universe.getCanvas().getScreen3D());
		transform=new TransformGroup(translation1);
		transform.addChild(map);
		
		platform=new ViewPlatform();
		map.attachViewPlatform(platform);
		birdView.addChild(platform);
		translation2=new Transform3D();
		translation2.rotX(-Math.PI*80.0/180.0);
		translation2.setTranslation(new Vector3d(0.0,7.0,0.2));
		birdView.setTransform(translation2);
		
		return transform;
	}

	private Node createBirdView()
	{
		TransformGroup transform;
		Behavior behavior;
		BoundingSphere bounds;
	
		transform=new TransformGroup();
		
		bounds=new BoundingSphere(new Point3d(),Double.POSITIVE_INFINITY);
		behavior=new FreeViewBehavior(
		    universe.getViewingPlatform().getViewPlatformTransform());
		behavior.setSchedulingBounds(bounds);
		transform.addChild(behavior);
		
		birdView=new TransformGroup();
		transform.addChild(birdView);
		
		return transform;
	}
}
